<?xml version="1.0" encoding="UTF-8" ?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml"/>
	<xsl:param name="param"/>
	<xsl:variable name="imgbase">/cron/pub/</xsl:variable>
	<xsl:variable name="imgbase2">/tools/introspect/pub/</xsl:variable>
	<xsl:variable name="sort"><xsl:value-of select="$param/param/sortby"/></xsl:variable>

    <xsl:template match="/jobs">
        <div xmlns:xrl="http://1060.org/xrl" >
			
			<script language="JavaScript"><xsl:comment>
				function doSubmit(name,action)
				{	document.main.name.value=name;
					document.main.action2.value=action;
					document.main.submit();
				}
			</xsl:comment></script>
					
			<form method="POST" name="main">
			<input type="hidden" name="sortby" value="{$sort}"/>
			<input type="hidden" name="name" value=""/>
			<input type="hidden" name="action2" value=""/>
			
			<table width="100%" class="data-table">
				<tr class="vis-title">
					<td>Name<br/>
						<xsl:call-template name="sortButtons">
							<xsl:with-param name="key" select="'name'"/>
						</xsl:call-template>
					</td>
					<td>Module/Request<br/>
						<xsl:call-template name="sortButtons">
							<xsl:with-param name="key" select="'module'"/>
						</xsl:call-template>
					</td>
					<td>Trigger Specification</td>
					<td>Next Scheduled<br/>
						<xsl:call-template name="sortButtons">
							<xsl:with-param name="key" select="'next'"/>
						</xsl:call-template>
					</td>
					<td/>
				</tr>
				
				<xsl:choose>
					<xsl:when test="count(job)=0">
						<tr>
							<td colspan="5">no jobs</td>
						</tr>
					</xsl:when>
					<xsl:when test="$sort = 'name'">
						<xsl:apply-templates select="job">
							<xsl:sort select="name" order="ascending" data-type="text"/>
						</xsl:apply-templates>
					</xsl:when>
					<xsl:when test="$sort = 'nameRev'">
						<xsl:apply-templates select="job">
							<xsl:sort select="name" order="descending" data-type="text"/>
						</xsl:apply-templates>
					</xsl:when>
					<xsl:when test="$sort = 'module'">
						<xsl:apply-templates select="job">
							<xsl:sort select="module" order="ascending" data-type="text"/>
						</xsl:apply-templates>
					</xsl:when>
					<xsl:when test="$sort = 'moduleRev'">
						<xsl:apply-templates select="job">
							<xsl:sort select="module" order="descending" data-type="text"/>
						</xsl:apply-templates>
					</xsl:when>
					<xsl:when test="$sort = 'next'">
						<xsl:apply-templates select="job">
							<xsl:sort select="next-raw" order="ascending" data-type="number"/>
						</xsl:apply-templates>
					</xsl:when>
					<xsl:when test="$sort = 'nextRev'">
						<xsl:apply-templates select="job">
							<xsl:sort select="next-raw" order="descending" data-type="number"/>
						</xsl:apply-templates>
					</xsl:when>
					<xsl:otherwise>
						<xsl:apply-templates select="job"/>
					</xsl:otherwise>
				</xsl:choose>
			</table>

			<div style="margin-top: 1em;">
				<input type="button" value="refresh" onclick="javascript:document.main.submit()" class="btn"/>
				<input type="button" value="view log" onclick="javascript:parent.location='/tools/log-view?log=org.netkernel.mod.cron'" class="btn"/>
				<input type="button" value="configure log" onclick="javascript:parent.location='/tools/log-levels'" class="btn"/>
			</div>

			</form>

		</div>
    </xsl:template>

	<xsl:template match="job">
		<xsl:variable name="class">
			<xsl:choose>
				<xsl:when test="paused">
					<xsl:text>disabledrow</xsl:text>
				</xsl:when>
				<xsl:when test="position() mod 2 = 0">
					<xsl:text>evenrow</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>oddrow</xsl:text>
				</xsl:otherwise>  
			</xsl:choose>
		</xsl:variable>
		<tr class="{$class}">
			<td>
				<div style="font-weight: bold;"><xsl:value-of select="name"/></div>
				<div style="font-size: 80%;"><xsl:value-of select="desc"/></div>
			</td>
			<td>
				<div><xsl:value-of select="space"/></div>
				<div>(<xsl:value-of select="version"/>)</div>
				<div>SOURCE <xsl:value-of select="resource"/></div>
			</td>
			
			<td ><xsl:value-of select="trigger"/></td>
			<td><xsl:value-of select="next"/></td>
			<td>
				<div style="white-space:nowrap;">
				<xsl:choose>
					<xsl:when test="paused">
						<a title="resume"><xsl:attribute name="href">javascript:doSubmit('<xsl:value-of select="name"/>','resume')</xsl:attribute>
							<img style="border: none; padding: 1px;" src="{$imgbase}resume.png"/>
						</a>
					</xsl:when>
					<xsl:otherwise>
						<a title="pause"><xsl:attribute name="href">javascript:doSubmit('<xsl:value-of select="name"/>','pause')</xsl:attribute>
							<img style="border: none; padding: 1px;" src="{$imgbase}pause.png"/>
						</a>
					</xsl:otherwise>
				</xsl:choose>
				<a title="delete"><xsl:attribute name="href">javascript:doSubmit('<xsl:value-of select="name"/>','delete')</xsl:attribute>
					<img style="border: none; padding: 1px;" src="{$imgbase}delete.png"/>
				</a>
				<a title="run now"><xsl:attribute name="href">javascript:doSubmit('<xsl:value-of select="name"/>','run')</xsl:attribute>
					<img style="border: none; padding: 1px;" src="{$imgbase}exec.png"/>
				</a>
				</div>
			</td>
			
		</tr>
	</xsl:template>
	
	<xsl:template name="sortButtons">
		<xsl:param name="key"/>
		<xsl:choose>
			<xsl:when test="$sort=$key">
					<a href="?sortby={$key}"><img class="sort" src="{$imgbase2}upSelected.png" alt="[^]"/></a>
			</xsl:when>
			<xsl:otherwise>
				<a href="?sortby={$key}"><img class="sort" src="{$imgbase2}up.png" alt="[^]"/></a>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:choose>
			<xsl:when test="$sort=concat($key,'Rev')">
					<a href="?sortby={$key}Rev"><img class="sort" src="{$imgbase2}downSelected.png" alt="[v]"/></a>
			</xsl:when>
			<xsl:otherwise>
				<a href="?sortby={$key}Rev"><img class="sort" src="{$imgbase2}down.png" alt="[v]"/></a>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>	
	
	
</xsl:stylesheet>
