package org.netkernel.mod.cron.services;

import java.util.Date;

import org.netkernel.container.IKernel;
import org.netkernel.layer0.nkf.INKFLocale;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.representation.impl.HDSFactory;
import org.netkernel.layer0.util.FastSchematron;
import org.netkernel.layer0.util.PairList;
import org.netkernel.layer0.util.RequestBuilder;
import org.netkernel.mod.cron.transport.CronTransport;
import org.netkernel.mod.cron.transport.NetKernelRequestJob;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.urii.ISpace;
import org.netkernel.urii.ISpaceWithIdentity;
import org.netkernel.util.Utils;
import org.quartz.*;
import org.w3c.dom.Document;

public class CronUtils
{
	private static final PairList VALIDATE_JOB = new PairList(4);
	static
	{	VALIDATE_JOB.put("/*[name()!='job']", "MSG_JOB_1");
		VALIDATE_JOB.put("/job/*[name()!='name' and name()!='desc' and name()!='request' and name()!='userJobData' and name()!='cron' and name()!='simple' ]", "MSG_JOB_2");
		VALIDATE_JOB.put("/job[count(cron | simple)>1]", "MSG_JOB_3");
		VALIDATE_JOB.put("/job/userJobData/*[name()!='data']", "MSG_JOB_4");
		VALIDATE_JOB.put("/job/userJobData/data[not(name) or not(value)]", "MSG_JOB_5");
		VALIDATE_JOB.put("/job/cron/*[name()!='expr' and name()!='startTime' and name()!='endTime' ]", "MSG_JOB_6");
		VALIDATE_JOB.put("/job/simple/*[name()!='startTime' and name()!='endTime' and name()!='repeatCount' and name()!='repeatInterval' ]", "MSG_JOB_7");
		VALIDATE_JOB.put("/job[not(name)]", "MSG_JOB_8");
	}
	
	public static void validateJob(IHDSNode aJob, INKFRequestContext aContext) throws Exception
	{	Document jobAsDom=HDSFactory.toDOM(aJob);
		PairList validation=FastSchematron.validate(jobAsDom.getDocumentElement(), VALIDATE_JOB);
		if (validation.size()>0)
		{	String m = FastSchematron.formatValidationError(validation,aContext);			
			throw aContext.createFormattedException("EX_JOB_VALIDATE",  "MSG_RAW", null, null, m);
		}
	}
		
	public static void addJob(IHDSNode aJob, String aSpace, String aVersion,String aType, Scheduler aScheduler, INKFRequestContext aContext) throws Exception
	{
		String name = aJob.getFirstValue("name").toString().trim();
		if (aType.equals(NetKernelRequestJob.KEY_TYPE_MISFIRE))
		{	name += "-misfire";
		}
		
		String desc = "";
		if (aJob.getFirstNode("desc")!=null)
		{	desc=aJob.getFirstValue("desc").toString().trim();
		}
		//String uri = aJob.getFirstValue("uri").toString().trim();
		
		Document reqDoc=HDSFactory.toDOM(aJob.getFirstNode("request"));
		RequestBuilder requestBuilder = new RequestBuilder(reqDoc,aContext.getKernelContext().getKernel().getLogger());
		IHDSNode rbHDS=HDSFactory.parseDOM(reqDoc);
		JobDetail jd = new JobDetail(name, Scheduler.DEFAULT_GROUP, NetKernelRequestJob.class);
		jd.getJobDataMap().put(NetKernelRequestJob.KEY_DESCRIPTION, desc);
		jd.getJobDataMap().put(NetKernelRequestJob.KEY_NAME, name);
		jd.getJobDataMap().put(NetKernelRequestJob.KEY_REQUEST, requestBuilder);
		jd.getJobDataMap().put(NetKernelRequestJob.KEY_REQUEST_DEF, rbHDS);
		jd.getJobDataMap().put(NetKernelRequestJob.KEY_SPACE, aSpace);
		jd.getJobDataMap().put(NetKernelRequestJob.KEY_VERSION, aVersion);
		jd.getJobDataMap().put(NetKernelRequestJob.KEY_TYPE, aType);

		for (IHDSNode dataNode: aJob.getNodes("userJobData/data"))
		{	String key = dataNode.getFirstValue("name").toString().trim();
			Object value=dataNode.getFirstValue("value");
			jd.getJobDataMap().put(key, value);
		}

		Trigger toSchedule;
		if (aType.equals(NetKernelRequestJob.KEY_TYPE_MISFIRE))
		{	toSchedule = new SimpleTrigger(name, Scheduler.DEFAULT_GROUP);
		}
		else if (aJob.getFirstNode("cron")!=null)
		{	IHDSNode cronNode=aJob.getFirstNode("cron");
			String expr = cronNode.getFirstValue("expr").toString().trim();
			long start = getLong(cronNode, "startTime");
			long end = getLong(cronNode, "endTime");
			long now = System.currentTimeMillis();
			long yearAgo = now - 31536000000L;
			if (start >= 0 && start < yearAgo)
			{	start += System.currentTimeMillis();
			}
			if (end >= 0 && end < yearAgo)
			{	end += System.currentTimeMillis();
			}
			CronTrigger t = new CronTrigger(name, Scheduler.DEFAULT_GROUP, expr);
			if (start >= 0)
			{	t.setStartTime(new Date(start));
			}
			if (end >= 0)
			{	t.setEndTime(new Date(end));
			}
			toSchedule=t;
		}
		else if (aJob.getFirstNode("simple")!=null)
		{	IHDSNode simpleNode=aJob.getFirstNode("simple");
			long start = getLong(simpleNode, "startTime");
			long end = getLong(simpleNode, "endTime");
			int repeatCount = (int)getLong(simpleNode, "repeatCount");
			long repeatInterval = getLong(simpleNode, "repeatInterval");
			long now = System.currentTimeMillis();
			long yearAgo = now - 31536000000L;
			if (start >= 0 && start < yearAgo)
			{	start += System.currentTimeMillis();
			}
			if (end >= 0 && end < yearAgo)
			{	end += System.currentTimeMillis();
			}
			if (start < 0 && end < 0 && repeatCount < 0 && repeatInterval < 0)
			{ // right now
				toSchedule = new SimpleTrigger(name, Scheduler.DEFAULT_GROUP);
			}
			else if (start < 0 && end < 0 && repeatInterval > 0)
			{ // interval and optional repeat
				if (repeatCount < 0)
				{	repeatCount = SimpleTrigger.REPEAT_INDEFINITELY;
				}
				toSchedule = new SimpleTrigger(name, Scheduler.DEFAULT_GROUP, repeatCount, repeatInterval);
			}
			else if (end < 0 && repeatInterval < 0 && repeatCount < 0)
			{ // once at specified time
				toSchedule = new SimpleTrigger(name, Scheduler.DEFAULT_GROUP, new Date(start));
			}
			else if (start >= 0 && repeatInterval > 0)
			{ // all the flags
				if (repeatCount < 0)
				{	repeatCount = SimpleTrigger.REPEAT_INDEFINITELY;
				}
				Date endDate;
				if (end < 0)
				{	endDate = null;
				}
				else
				{	endDate = new Date(end);
				}
				Date startDate = new Date(start);
				toSchedule = new SimpleTrigger(name, Scheduler.DEFAULT_GROUP, startDate, endDate, repeatCount, repeatInterval);
			}
			else
			{	throw aContext.createFormattedException("EX_CRON_BAD_TRIGGER", "MSG_CRON_BAD_SIMPLE_TRIGGER", null, null, name);
			}
		}
		else
		{	throw aContext.createFormattedException("EX_CRON_BAD_TRIGGER", "MSG_CRON_BAD_TRIGGER_TYPE", null, null, name);
		}
		aScheduler.scheduleJob(jd, toSchedule);
		aContext.logFormatted(INKFLocale.LEVEL_INFO, "MSG_CRON_ADDED_JOB", name,aType);
	}
	
	public static long getLong(IHDSNode aNode, String aPath)
	{	IHDSNode n=aNode.getFirstNode(aPath);
		long result=-1L;
		if (n!=null && n.getValue()!=null)
		{	try
			{	String string = n.getValue().toString().trim();
				return Long.parseLong(string);
			}
			catch (Exception e)	{	}
		}
		return result;
	}
	
	public static JobDetail getJob(String aName, Scheduler aScheduler) throws SchedulerException
	{	String[] groups = aScheduler.getJobGroupNames();
		for (int g = 0; g < groups.length; g++)
		{	String[] jobNames = aScheduler.getJobNames(groups[g]);
			for (int j = 0; j < jobNames.length; j++)
			{	String jobName = jobNames[j];
				if (jobName.equals(aName))
				{	JobDetail jobDetail = aScheduler.getJobDetail(jobName,	groups[g]);
					return jobDetail;
				}
			}
		}
		return null;
	}	
	
	public static ISpaceWithIdentity getInjectionSpace(INKFRequestContext aContext, int aSkipCount)
	{
		IRequestScopeLevel scope=aContext.getKernelContext().getRequestScope();
		for (int i=0; i<aSkipCount && scope.getParent()!=null; i++)
		{	scope=scope.getParent();
		}
		IKernel k=aContext.getKernelContext().getKernel();
		ISpaceWithIdentity result=null;
		while(scope!=null)
		{	ISpace space= scope.getSpace();
			if (space instanceof ISpaceWithIdentity)
			{	ISpaceWithIdentity sid=(ISpaceWithIdentity)space;
				if (k.getSpace(sid.getIdentifier(), sid.getVersion(), sid.getVersion())!=null)
				{	result=sid;
					break;
				}
			}
			scope=scope.getParent();
		}
		return result;
	}
	
	public static IHDSNode getState(Scheduler s) throws SchedulerException
	{
		HDSBuilder b = new HDSBuilder();
		b.pushNode("jobs");
		String[] jobNames=s.getJobNames(Scheduler.DEFAULT_GROUP);
        for (int j=0; j<jobNames.length; j++)
        {	String jobName=jobNames[j];
            JobDetail jobDetail=s.getJobDetail(jobName,Scheduler.DEFAULT_GROUP);
            JobDataMap jdm=jobDetail.getJobDataMap();
            Trigger trigger=s.getTriggersOfJob(jobName,Scheduler.DEFAULT_GROUP)[0];
            if (trigger!=null)
            {	b.pushNode("job");
	            b.addNode("name",jobName);
	            b.addNode("desc",jdm.getString(NetKernelRequestJob.KEY_DESCRIPTION));
	            b.addNode("space",jdm.getString(NetKernelRequestJob.KEY_SPACE));
	            b.addNode("version",jdm.getString(NetKernelRequestJob.KEY_VERSION));
	            //RequestBuilder rb=(RequestBuilder)jdm.get(NetKernelRequestJob.KEY_REQUEST);
	            IHDSNode jobDef=(IHDSNode)jdm.get(NetKernelRequestJob.KEY_REQUEST_DEF);
	            try
	            {	//IHDSNode rbHDS=HDSFactory.parseDOM(rb.getConfiguration());
	            	b.importChildren(jobDef);
	            }
	            catch (Exception e)
	            {	//should not happen as there are no types
	            }
				b.addNode("type",jdm.getString(NetKernelRequestJob.KEY_TYPE));
	            Date nextTime=trigger.getNextFireTime();
				if (nextTime!=null)
				{	b.addNode("next",nextTime.toString());
					b.addNode("next-raw",Long.toString(nextTime.getTime()));
				}
				Date finalTime=trigger.getFinalFireTime();
				if (finalTime!=null)
				{	b.addNode("final",Long.toString(finalTime.getTime()));
				}
	            
				if(s.getTriggerState(trigger.getName(), trigger.getGroup())==Trigger.STATE_PAUSED)
	            {	b.addNode("paused","true");
	            }
	            
				b.pushNode("userJobData");
				
	            String[] keys=jdm.getKeys();
	            for (int i=0; i<keys.length; i++)
	            {	if (!keys[i].startsWith(NetKernelRequestJob.KEY_NK_PREFIX))
	            	{	b.pushNode("data");
	                	b.addNode("name",keys[i]);
	                	b.addNode("value",jdm.getString(keys[i]));
	                	b.popNode();
	            	}
	            }
	            b.popNode();
	            
				if (trigger instanceof CronTrigger)
				{	b.pushNode("cron");
					b.addNode("expr",((CronTrigger)trigger).getCronExpression());
					if (trigger.getStartTime()!=null)
					{  b.addNode("startTime", Long.toString(trigger.getStartTime().getTime()));
					}
					if (trigger.getEndTime()!=null)
					{   b.addNode("endTime", Long.toString(trigger.getEndTime().getTime()));
					}
					b.popNode();
				}
				else if (trigger instanceof SimpleTrigger)
				{	b.pushNode("simple");
					SimpleTrigger st=(SimpleTrigger)trigger;
					if (trigger.getStartTime()!=null)
					{  b.addNode("startTime", Long.toString(trigger.getStartTime().getTime()));
					}
					if (trigger.getEndTime()!=null)
					{   b.addNode("endTime", Long.toString(trigger.getEndTime().getTime()));
					}			
					if (st.getRepeatCount()>0)
					{   b.addNode("repeatCount", Long.toString(st.getRepeatCount()-st.getTimesTriggered()));
					}
					if (st.getRepeatInterval()>0)
					{	b.addNode("repeatInterval", Long.toString(st.getRepeatInterval()));
					}
					b.popNode();
				}
	            
	            StringBuilder sb=new StringBuilder(128);
	            if (trigger instanceof CronTrigger)
	            {	CronTrigger ct=(CronTrigger)trigger;
	                sb.append("Cron [");
	                sb.append(ct.getCronExpression());
	                if (trigger.getStartTime()!=null)
	                {   sb.append(" start=");
	                    sb.append(trigger.getStartTime().toString());
	                }
	                if (trigger.getEndTime()!=null)
	                {   sb.append(" end=");
	                    sb.append(trigger.getEndTime().toString());
	                }
	                sb.append(" ]");
	            }
	            else if (trigger instanceof SimpleTrigger)
	            {	SimpleTrigger st=(SimpleTrigger)trigger;
	                sb.append("Simple [");
	                if (trigger.getStartTime()!=null)
	                {   sb.append("start=");
	                    sb.append(trigger.getStartTime().toString());
	                }
	                if (trigger.getEndTime()!=null)
	                {   sb.append(" end=");
	                    sb.append(trigger.getEndTime().toString());
	                }
	                sb.append(" count=");
	                if (st.getRepeatCount()>=0)
	                {   sb.append(st.getRepeatCount());
	                }
	                else
	                {   sb.append("INDEFINITELY");
	                }
	                sb.append(" interval=");
	                sb.append(st.getRepeatInterval()/1000);
	                sb.append("(s)");
	
	                sb.append("]");
	            }
	            b.addNode("trigger",sb.toString());
	            b.popNode();
            }
        }
	    b.popNode();
	    return b.getRoot();
	}
	
}
