/******************************************************************************
 * (c) Copyright 2002,2005, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile: CronTransport.java,v $
 * Version:       $Name:  $ $Revision: 1.13 $
 * Last Modified: $Date: 2007/12/03 10:16:23 $
 *****************************************************************************/
package org.netkernel.mod.cron.transport;

import java.sql.Date;

import org.netkernel.layer0.boot.BootPersistence;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.nkf.impl.NKFLifecycleEventContextImpl;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.representation.impl.HDSFactory;
import org.netkernel.layer0.representation.impl.HDSPredicateFactory;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.util.RequestBuilder;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.mod.cron.services.CronUtils;
import org.netkernel.module.standard.endpoint.StandardTransportImpl;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestResponseFieldsImpl;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.ISpace;
import org.netkernel.urii.impl.Version;
import org.netkernel.util.Utils;
import org.quartz.*;
import org.quartz.impl.StdSchedulerFactory;
import org.w3c.dom.Document;

/**
 *
 * @author  Tony Butterfield
 */
public class CronTransport extends StandardTransportImpl
{
	private Scheduler mScheduler;
	private static String CRON_PERSISTENCE_RESOURCE="cron.xml";
	private static String CRON_INTERNAL_GROUP="CronTransport internal jobs";
	private static String CRON_POLL="crontab expiry poll";
	
	private static CronTransport sSingleton;
	
	private INKFResponseReadOnly mCrontabResponse;
	
	public CronTransport()
	{	sSingleton=this;
	}
	
	public void postCommission(INKFRequestContext aContext) throws Exception
	{	sSingleton=this;
		
		NKFLifecycleEventContextImpl logCtx = new NKFLifecycleEventContextImpl(getKernel(), aContext.getKernelContext().getRequestScope(), this, getParameters());
		LogBridge.init(logCtx);
	
		IReadableBinaryStreamRepresentation quartzProps = aContext.source("res:/etc/quartz.properties",IReadableBinaryStreamRepresentation.class);
		StdSchedulerFactory schedFact = new StdSchedulerFactory();
		schedFact.initialize(quartzProps.getInputStream());
		mScheduler=schedFact.getScheduler();
		aContext.logFormatted(INKFLocale.LEVEL_INFO, "MSG_START");
		mScheduler.start();
		
		loadPersistedState(aContext);
		
		//setup contab polling job - delay startup by 10 seconds
		JobDetail jd = new JobDetail(CRON_POLL,CRON_INTERNAL_GROUP,CronPollJob.class);
		jd.getJobDataMap().put(CronPollJob.KEY_TRANSPORT, this);
		long pollPeriod=aContext.getKernelContext().getKernel().getConfiguration().getLong("netkernel.poll");
		Date start=new Date(System.currentTimeMillis()+10000L);
		Date end=new Date(System.currentTimeMillis()+60000L*12*365*10);
		SimpleTrigger t=new SimpleTrigger(CRON_POLL, CRON_INTERNAL_GROUP,start,end,SimpleTrigger.REPEAT_INDEFINITELY,pollPeriod);
		mScheduler.scheduleJob(jd, t);
	}
	
	private void loadPersistedState(INKFRequestContext aContext)
	{	IReadableBinaryStreamRepresentation rbs;
		try
		{	rbs=BootPersistence.sourceBootResource(CRON_PERSISTENCE_RESOURCE, aContext.getKernelContext().getKernel());
		}
		catch (Exception e)
		{	return;
		}
		try
		{	aContext.logFormatted(INKFLocale.LEVEL_INFO, "MSG_LOAD");
			IHDSNode state=HDSFactory.parseXML(rbs.getInputStream());
			long now=System.currentTimeMillis();
			for(IHDSNode jobNode : state.getNodes("/jobs/job"))
			{
				long next=Long.parseLong(jobNode.getFirstValue("next-raw").toString());
				if (next<now)
				{	String name=jobNode.getFirstValue("name").toString();
					aContext.logFormatted(INKFLocale.LEVEL_WARNING, "MSG_MISSED",name);
					try
					{	String space=jobNode.getFirstValue("space").toString();
						String version=jobNode.getFirstValue("version").toString();
						//schedule now
						CronUtils.addJob(jobNode, space, version, NetKernelRequestJob.KEY_TYPE_MISFIRE, getScheduler(), aContext);
						
		                long finalTime=CronUtils.getLong(jobNode,"final");
		                String type=jobNode.getFirstValue("type").toString();
		                if (type.equals(NetKernelRequestJob.KEY_TYPE_DIRECT) && (finalTime<0 || finalTime>now))
		                {   // only add the direct jobs back in
		                    // we'll load the entrypoint ones afresh
		                    CronUtils.addJob(jobNode, space, version, type, getScheduler(), aContext);
		                }
					}
					catch (Exception e)
					{	aContext.logFormatted(INKFLocale.LEVEL_WARNING, "MSG_LOAD_ERROR",name,Utils.throwableToString(e));
					}
				}
			}
			
		}
		catch (Exception e)
		{	
			aContext.logFormatted(INKFLocale.LEVEL_WARNING, "MSG_RAW",Utils.throwableToString(e));
		}
		finally
		{	try
			{	BootPersistence.deleteBootResource(CRON_PERSISTENCE_RESOURCE, aContext.getKernelContext().getKernel());
			}
			catch (Exception e2)
			{ }
		}
	}
	
	void pollCrontab()
	{	if (mCrontabResponse==null || mCrontabResponse.isExpired())
		{	INKFRequestContext ctx=getTransportContext();
			try
			{	
				//remove existing crontab jobs
				Scheduler s = getScheduler();
				String[] jobNames=s.getJobNames(Scheduler.DEFAULT_GROUP);
		        for (int j=0; j<jobNames.length; j++)
		        {	String jobName=jobNames[j];
		            JobDetail jobDetail=s.getJobDetail(jobName,Scheduler.DEFAULT_GROUP);
		            JobDataMap jdm=jobDetail.getJobDataMap();
					if  (jdm.getString(NetKernelRequestJob.KEY_TYPE).equals(NetKernelRequestJob.KEY_TYPE_CRONTAB))
					{	s.deleteJob(jobName,Scheduler.DEFAULT_GROUP);
					}
		        }
		        
		        if (mCrontabResponse==null)
				{	ctx.logFormatted(INKFLocale.LEVEL_INFO, "MSG_CRONTAB_LOAD");
				}
				else
				{	ctx.logFormatted(INKFLocale.LEVEL_INFO, "MSG_CRONTAB_CHANGE");
				}
				
				INKFRequest req=ctx.createRequest("active:spaceAggregateHDS");
				req.addArgument("uri", "res:/etc/system/CronConfig.xml");
				req.setRepresentationClass(IHDSNode.class);
				mCrontabResponse=ctx.issueRequestForResponse(req);
				
				IHDSNode n=(IHDSNode)mCrontabResponse.getRepresentation();
				IHDSNodeList cronSpaces=n.getNodes("/spaces/space").filter(HDSPredicateFactory.hasChild("jobs"));
				for(IHDSNode cronNode : cronSpaces)
				{	String space=cronNode.getFirstValue("id").toString().trim();
					String version=cronNode.getFirstValue("version").toString().trim();
					IHDSNode cronConfigNode=cronNode.getFirstNode("jobs");
					for (IHDSNode jobNode: cronConfigNode.getChildren())
					{	try
						{	CronUtils.validateJob(jobNode, ctx);
							CronUtils.addJob(jobNode,space,version, NetKernelRequestJob.KEY_TYPE_CRONTAB,getScheduler(),ctx);
						}
						catch (Exception e)
						{	ctx.logFormatted(INKFLocale.LEVEL_WARNING, "MSG_CRONTAB_ERROR",space,Utils.throwableToString(e));
						}
					}
				}
			}
			catch (Exception e)
			{	ctx.logFormatted(INKFLocale.LEVEL_WARNING, "MSG_RAW",Utils.throwableToString(e));
			}
		}
	}
	
	
	public void preDecommission(INKFRequestContext aContext) throws Exception
	{	mScheduler.standby();
		aContext.logFormatted(INKFLocale.LEVEL_INFO, "MSG_STOP");
		
		//save jobs
		try
		{	IHDSNode state=CronUtils.getState(mScheduler);
			Document d=HDSFactory.toDOM(state);
			String s=XMLUtils.toXML(d, true, false);
			ByteArrayRepresentation rep=new ByteArrayRepresentation(s.getBytes("UTF-8"));
			BootPersistence.sinkBootResource(CRON_PERSISTENCE_RESOURCE, rep, getKernel());
			aContext.logFormatted(INKFLocale.LEVEL_INFO, "MSG_SAVED");
		}
		catch (Exception e)
		{	aContext.logFormatted(INKFLocale.LEVEL_WARNING, "MSG_SAVE_ERROR",Utils.throwableToString(e));
		}

		sSingleton=null;
		mScheduler.shutdown();
		mScheduler=null;
	}
	
	void handleRequest(String aName, RequestBuilder aReqBuilder, String aSpace, String aVersion) throws NKFException
	{	INKFRequestContext context=getTransportContext();
		context.logFormatted(INKFLocale.LEVEL_FINE, "MSG_RUNNING_JOB", aName);
		try
		{	
			INKFRequest req=aReqBuilder.buildRequest(context, null, null);
			
			IIdentifier sid=new SimpleIdentifierImpl(aSpace);
			Version v=null;
			if (aVersion!=null)
			{	v=new Version(aVersion);
			}
			ISpace space=context.getKernelContext().getKernel().getSpace(sid, v, v);
			if (space!=null)
			{	IRequestScopeLevel scope = RequestScopeLevelImpl.createOrphanedRootScopeLevel(space, context.getKernelContext().getRequestScope());
				req.setRequestScope(scope);	
				INKFAsyncRequestHandle handle = context.issueAsyncRequest(req);
				handle.setListener(new JobListener(aName));
			}
		}
		catch (Exception e)
		{	context.logFormatted(INKFLocale.LEVEL_WARNING, "MSG_JOB_EXEC_ERROR", Utils.throwableToString(e));
		}
	}
	
	public Scheduler getScheduler()
	{	if (mScheduler==null)
		{	throw new NullPointerException("Scheduler has not been successfully initialised");
		}
		return mScheduler;
	}
	
	public static CronTransport getSingleton()
	{	return sSingleton;
	}
	
	public IRequestResponseFields getState()
	{	RequestResponseFieldsImpl state=new RequestResponseFieldsImpl(super.getState());
		try
		{	state.put("scheduler", CronUtils.getState(mScheduler));
		}
		catch (SchedulerException e) {;}
		return state;
	}
	

	
	private class JobListener implements INKFAsyncRequestListener
	{
		private final String mJobName;
		
		public JobListener(String aJobName)
		{	mJobName=aJobName;
		}
		
		public void receiveException(NKFException aException, INKFRequest aRequest, INKFRequestContext aContext) throws Exception
		{	
			if (aException.getDeepestId().equals("Uncommissioned Space"))
			{	// ignore this error because it because module was restarted whilst cron job execution was pending
			}
			else
			{	aContext.logFormatted(INKFLocale.LEVEL_WARNING, "MSG_JOB_EXEC_ERROR", mJobName,Utils.throwableToString(aException));
			}
		}

		@SuppressWarnings("unchecked")
		public void receiveResponse(INKFResponseReadOnly aResponse, INKFRequestContext aContext) throws Exception
		{	Object rep=aResponse.getRepresentation();
			if (rep!=null)
			{	aContext.logRaw(INKFLocale.LEVEL_FINER, rep.toString());
			}
		}
	}
	
	public static class CronPollJob implements Job
	{
		public static final String KEY_TRANSPORT="transport";
		
		public void execute(JobExecutionContext aJEC) throws JobExecutionException
		{
			CronTransport transport = (CronTransport)aJEC.getJobDetail().getJobDataMap().get(KEY_TRANSPORT);
			transport.pollCrontab();
		}
	}
	
}